//_________________________________________________________________________________-_______________________
// OakLibTest_noHotplug.cpp
// 
// This is a very simple console application that demonstrates how to communicate with the Oak sensors,
// using the Oak library provided by Toradex.
// 
// It was tested in Visual Studio 2005 running on Windows XP
//
// June 29, 2007, Andy Kiser, Toradex AG
//________________________________________________________________________________________________________
#include <windows.h>
#include <stdio.h>
#include <tchar.h>
#include "Oak.h"

#ifdef _UNICODE
#error ASCII mode required: check project properties -> Configuration Properties -> General -> Character Set
#endif

// __________________________ O a k _ P r i n t S e n s o r I n f o ______________________________________
// Print some information that is provided by the Oak Library during the initialization of the sensor
// (This routine contains no USB specific code, it just prints data)

void Oak_PrintSensorInfo(PtOakSensor Sensor)
{
	printf_s("===============================================\n");
	printf_s("%s (usr:%s)\n", Sensor->DeviceName, Sensor->UserDeviceName);
	printf_s("===============================================\n\n");
	printf_s("VID:%04X  PID:%04X  REV:%04X  SN:%s\n", Sensor->VID, Sensor->PID, Sensor->REV, Sensor->SN);
	printf_s("\nDevicePath:\n%s\n", Sensor->DevicePath);

	printf ("\n  %i Data Channels\n", Sensor->NumChannels);
	for (int ch=0; ch<Sensor->NumChannels; ch++)
	{
		printf_s("\n  ----------\n");
		printf_s("  Channel %i\n", ch);
		printf_s("  ----------\n");
		printf_s("  %s (usr:%s)\n", Sensor->Channel[ch].ChannelName, Sensor->Channel[ch].UserChannelName);
		if (Sensor->Channel[ch].IsSigned)
			printf_s("  %i Bit signed\n", Sensor->Channel[ch].BitSize);
		else
			printf_s("  %i Bit unsigned\n", Sensor->Channel[ch].BitSize);
		printf("  Resolution: 1e%i %s\n", Sensor->Channel[ch].UnitExponent, Sensor->Channel[ch].UnitStr);
	}
}

// __________________________ _t m a i n _________________________________________________________________
// this is already the main function.
// As you see, I tried to keep the complexity of the application low, so you can easily find what this 
// application wants to show: communication with the Oak sensor.

int _tmain(int argc, _TCHAR* argv[])
{
	bool		fSuccess;
	tOakSensor	Sensor;			// details about the Oak sensor and its data channels (name, data format, ...)
	BYTE		InReport[33];	// USB data is transferred in reports.

	// initialization: Find one specific sensor
	// Note: this is a basic application that only reacts on one Oak Sensor, even if multiple are connected

	fSuccess = Oak_FindSensor( 0x0000,	// search for ANY PID (product ID)
							   0x0000,	// search for ANY REV (product revision)
							   NULL,	// search for ANY serial number
							   NULL,	// search for ANY device name
							   NULL,	// search for ANY channel name
							   NULL,	// search for ANY user defined device name
							   NULL,	// search for ANY user defined channel name
							   &Sensor); // <- store inforation about the first Oak sensor that matches 
										 //    the criteria above into this variable

	if (!fSuccess)	// no matching sensor found --> just leave
		return -1;

	// Just for fun: print some information about the sensor we have just found
	Oak_PrintSensorInfo(&Sensor);
	
	// Read current sensor values
	// This is one of two similar commands (actually the depreceated one)
	// It bypasses the operating system's report buffer and instead directly triggers USB communication through
	// the so-called USB control pipe, which lacks realtime capabilities
	// This is *not* supported for Windows systems before XP !
	
	fSuccess = Oak_GetCurrentInReport(Sensor.DevicePath, InReport, sizeof(InReport));
	if (!fSuccess)
		printf_s("/* ERROR: GetReport failed */\n"); // this should never happen
	else
	{
		// InReport now contains the current sensor values in raw format.
		// Let's keep it simple and print them in raw hex format.
		// (By now all Oak sensors provide their values in 16 bits width)

		printf_s("\nRaw Sensor Values (unbuffered):\n");
		for (int ch=0; ch<Sensor.NumChannels; ch++)
			printf_s("Channel %i: 0x%04X\n", ch, InReport[1+2*ch] + 256*InReport[2+2*ch]);
	}

	// Read current sensor values
	// Ok, this time we use the second of the similar commands. You have probably seen, there are two additional
	// Parameters:
	// Param 3 is the report size. It has to match exactly, otherwise we do not get anything. Remember, by now
	//         every channel data is 16 bits (2 bytes); in addition there is one preceding byte that contains the 
	//         report number (this is always 0)
	// Param 4 is a timeout value
	//         In contrast to the method above, this function reads every value reported by the sensor device exactly once.
	//         If there is no updated value available within 1933ms in this example, the function returns indicating a failure.
	fSuccess = Oak_GetInReport(Sensor.DevicePath, InReport, 1+2*Sensor.NumChannels, 1933);
	if (!fSuccess)
		printf_s("/* ERROR: ReadFile failed */\n");
	else
	{
		// exactly the same as above...
		printf_s("\nRaw Sensor Values (buffered):\n");
		for (int ch=0; ch<Sensor.NumChannels; ch++)
			printf_s("Channel %i: 0x%04X\n", ch, InReport[1+2*ch] + 256*InReport[2+2*ch]);
	}
	
	// Just wait for the <ENTER> key, before closing the window
	char ch = getchar();
	return 0;
}

